## jdbc_mysql

Démonstration d'une base de données simple, avec le serveur MYSQL. La base de donnée est "dockerisée" et ne nécessite donc pas d'installation particulière.

Cet exemple est uniquement technique. Il ne prétend en particulier pas proposer de « bonnes pratiques ». On constatera par exemple qu'il stocke en dur les informations de connexion, ce qui est bien évidemment une abomination.

Le système utilise **docker**. Pour éviter d'éventuel problème avec une "vraie" base mysql qui serait installée sur votre machine, nous avons configuré le `docker-compose` pour que le port utilisé soit 3312 et non 3306. Vous pouvez cependant le modifier à votre guise (attention, il faut changer le `docker-compose.yml` **et** la classe `ConnectionManager.java`)

## Lancement de la base de donnée

- Installez [Docker](https://www.docker.com/) ;
- vérifiez que le service docker est bien lancé (ça dépend de votre installation). Lancez-le si ça n'est pas le cas.
- en ligne de commande, lancez 

    ~~~bash
    docker compose -f docker-compose.yml up
    ~~~

(le `-f docker-compose.yml`) n'est pas obligatoire ; il permet simplement de choisir le nom de son fichier de configuration ; par défaut c'est effectivement `docker-compose.yml`.

À la première exécution, docker chargera une image unix de `mariadb`, la version libre de `mysql`.

La configuration demande aussi au système d'initialiser la base avec le contenu du dossier `sql`.

Pour vérifier que tout va bien vous pouvez télécharger [DBeaver](https://dbeaver.io/) et consulter directement la base avec les paramètres :

url
: jdbc:mysql://localhost:3312/demo

login
: root

password
: root

Une fois que c'est fait, l'application peut être compilée (les tests utilisent la "vraie" base pour travailler) :

~~~shell
./gradlew build
~~~

- L'application se lance sans problème dans VSCode (nous ne testons pas sur les autres IDEs, mais elle a pas mal de chances de fonctionner) ;
- elle est lançable avec `gradle run` ; on conseille d'utiliser la commande :

        ./gradlew run --console=plain

- le `gradle build` construit deux archives qui contiennent des exécutables dans `build/distributions/` ; une fois décompressés ils devraient fonctionner.


## Remarques sur *docker*

Notez que notre `docker-compose.yml` est là pour vous simplifier la vie pendant des tests. Le contenu de la base est stocké dans l'image, ce qui fait que si vous arrêtez le serveur avec la commande 

~~~shell
docker compose -f docker-compose.yml down
~~~

le contenu de la base sera perdu.

Pour conserver celui-ci sur du long terme, il faudra définir un *volume* persistant, ce qui est très simple :

~~~yaml
version: "3.9"
services:
  db:
    image: mariadb:10.8.3-jammy
    restart: always
    ports:
      - 3312:3306
    environment:
      MARIADB_ROOT_PASSWORD: root
      MARIADB_DATABASE: demo          
    volumes:
      - ./sql:/docker-entrypoint-initdb.d # pour l'initialisation
      - data:/var/lib/mysql # pour la persistance après docker compose down.
volumes:
  data: # le volume persistant
~~~

C'est ce que fait le script `docker-compose-persistant.yml`.

## Docker et base de test

C'est évidemment une mauvaise idée d'utiliser la base de production (voire même la base de développement) pour les tests automatisés. Nous ne nous en occupons pas dans cet exemple, mais c'est en fait assez simple : on a deux configuration de connexion, l'une pour le logiciel, l'autre pour les tests.

Pour la base de tests :

- soit on utilise une base en mémoire, de type `h2`, mais le code exécuté ne sera a priori pas le même que pour la base de production (sauf évidemment si vous utilisez h2 en production, ce qui est possible mais rare) ;
- soit on configure deux conteneurs dockers, dont un pour le test.

## Tests et base de données

On peut évidemment écrire des tests du type :

- ma dao écrit une entrée
- je peux la relire

Mais si on désire vérifier plus finement l'interaction entre la base SQL et le code, il est pratique d'utiliser
des bibliothèques spécialisées, comme **DBUnit** (avec Spring, nous disposerons d'outils intégrés).

**DBUnit** dans son état actuel comporte beaucoup d'éléments spécifiques à JUnit 4. En gros, tout ce qui fonctionne par héritage. Nous avons quand même décidé de l'utiliser pour montrer comment on pouvait le mettre en œuvre.


L'intérêt de **DBUnit** est qu'il permet facilement de comparer le contenu attendu d'une table et le contenu calculé, en négligeant éventuellent certaines colonne. Voyons notre code ici :

~~~java
@Test
public void testInsert() throws SQLException, DatabaseUnitException {
  // Code spécifique à notre test :
  Connection connection = getJDBCConnection();
  PersonneDAO dao = new PersonneDAO(connection);
  dao.save(new Person(2, "Toto"));
  
  // Vérification du résultat avec DBUnit :
  // Canal pour récupérer les données :
  IDataSet databaseDataSet = getConnection().createDataSet();
  // On veut le contenu de la table "person"
  ITable actualTable = databaseDataSet.getTable("person");

  // Données attendues
  String xml = """
          <?xml version='1.0' encoding='UTF-8'?>
          <dataset>
              <person id="2" name="Toto"/>
          </dataset>
              """;
  IDataSet expectedDataSet = new FlatXmlDataSetBuilder().build(new StringReader(xml));
  ITable expectedTable = expectedDataSet.getTable("person");

  // On compare.
  Assertion.assertEquals(expectedTable, actualTable);
}
~~~

**DBUnit** repose sur un certain nombre de conventions :

- la base utilisée sert uniquement aux tests
- chaque test est responsable du nettoyage de son espace de travail **avant** de se lancer. En gros, 
  au lieu de tenter de remettre la base à son état initial et de faire un "rollback", le test va placer les tables sur lesquelles il travaille dans un état donné.

La mise en place du `IDatabaseTester` a été particulièrement pénible : la documentation est très mince (le système a été conçu pour JUnit 4 ; avec JUnit 5 on est un peu seul).

Le point le plus embêtant a été de remettre la base à "vide" avant chaque test (sans faire à la main le delete, c'eût été triché... même si c'était sans doute le plus rationnel ici).

Bref, point important : pour déclarer qu'une table est vide dans **DBUnit**, il faut écrire quelque chose comme :

~~~xml
<?xml version='1.0' encoding='UTF-8'?>
<dataset>
    <person/>
</dataset>
~~~

Vu nos exemple, **DBUnit** est parfaitement inutile. Mais avec des données plus complexes, il simplifie la vérification du contenu des bases.