package glg203.exemplemysql;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;

import java.io.InputStream;
import java.io.StringReader;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Collections;
import java.util.List;

import org.dbunit.Assertion;
import org.dbunit.DatabaseUnitException;
import org.dbunit.IDatabaseTester;
import org.dbunit.JdbcDatabaseTester;
import org.dbunit.database.IDatabaseConnection;
import org.dbunit.dataset.DataSetException;
import org.dbunit.dataset.IDataSet;
import org.dbunit.dataset.ITable;
import org.dbunit.dataset.xml.FlatXmlDataSetBuilder;
import org.dbunit.operation.DatabaseOperation;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

public class PersonneDAOTest {

    /**
     * Pour faire des tests DBUnit.
     */
    IDatabaseTester databaseTester;

    @BeforeEach
    public void initConnection() throws Exception {
        ConnectionManager manager = new ConnectionManager();
        databaseTester = new JdbcDatabaseTester(manager.getDriverName(),
                manager.getUrl(), manager.getUser(), manager.getPassword());
       
        databaseTester.setSetUpOperation(DatabaseOperation.CLEAN_INSERT);
        databaseTester.setDataSet(getEmptyDataset());
        databaseTester.onSetup();
    }

    // Important... nettoyage
    @AfterEach
    public void closeDatabasse() throws Exception {
        System.err.println("Closing"); // Should be a logging command.
        databaseTester.onTearDown();
        getConnection().close();
    }

    /**
     * Checks that findAll returns an empty list when there is no data.
     * 
     * @throws Exception
     */
    @Test
    public void checkEmptyDatabase() throws Exception {
        List<Person> persons = new PersonneDAO(getConnection().getConnection()).findAll();
        assertEquals(Collections.emptyList(), persons);
    }

    @Test
    public void testInsert1() throws SQLException, DatabaseUnitException {
        Connection connection = getJDBCConnection();
        PersonneDAO dao = new PersonneDAO(connection);
        dao.save(new Person(2, "Toto"));

        // Fetch database data after executing your code
        IDataSet databaseDataSet = getConnection().createDataSet();
        ITable actualTable = databaseDataSet.getTable("person");

        InputStream xmlStream = getRessource("toto.xml");
        assertNotNull(xmlStream);
        // Load expected data from an XML dataset
        IDataSet expectedDataSet = new FlatXmlDataSetBuilder().build(
                xmlStream);
        ITable expectedTable = expectedDataSet.getTable("person");

        // Assert actual database table match expected table
        Assertion.assertEquals(expectedTable, actualTable);
    }

    @Test
    // Même test que précédemment... vérifie l'indépendance des tests entre eux.
    // Ici, au lieu d'aller chercher le XML dans un fichier, on l'écrit directement.
    public void testInsert1Bis() throws SQLException, DatabaseUnitException {
        Connection connection = getJDBCConnection();
        PersonneDAO dao = new PersonneDAO(connection);
        dao.save(new Person(2, "Toto"));

        // Fetch database data after executing your code
        IDataSet databaseDataSet = getConnection().createDataSet();
        ITable actualTable = databaseDataSet.getTable("person");

        String xml = """
                <?xml version='1.0' encoding='UTF-8'?>
                <dataset>
                    <person id="2" name="Toto"/>
                </dataset>
                    """;
        IDataSet expectedDataSet = new FlatXmlDataSetBuilder().build(new StringReader(xml));
        ITable expectedTable = expectedDataSet.getTable("person");

        // Assert actual database table match expected table
        Assertion.assertEquals(expectedTable, actualTable);
    }

    @Test
    public void testInsertDeuxInsert() throws SQLException, DatabaseUnitException {
        Connection connection = getJDBCConnection();
        PersonneDAO dao = new PersonneDAO(connection);
        dao.save(new Person(5, "Titi"));
        dao.save(new Person(2, "Toto"));

        // Fetch database data after executing your code
        IDataSet databaseDataSet = getConnection().createDataSet();
        ITable actualTable = databaseDataSet.getTable("person");

        String xml = """
                <?xml version='1.0' encoding='UTF-8'?>
                <dataset>
                    <person id="2" name="Toto"/>
                    <person id="5" name="Titi"/>
                </dataset>
                    """;
        IDataSet expectedDataSet = new FlatXmlDataSetBuilder().build(new StringReader(xml));
        ITable expectedTable = expectedDataSet.getTable("person");

        // Assert actual database table match expected table
        Assertion.assertEquals(expectedTable, actualTable);
    }

    

    // Méthodes utilitaires

    public IDatabaseConnection getConnection() {
        try {
            return databaseTester.getConnection();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    public Connection getJDBCConnection() {
        try {
            return getConnection().getConnection();
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
    }

    public InputStream getRessource(String name) {
        return PersonneDAOTest.class.getResourceAsStream(name);
    }

    private IDataSet getEmptyDataset() {
        try {
            String xml = """
                    <?xml version='1.0' encoding='UTF-8'?>
                    <dataset>
                        <person/>
                    </dataset>""";
            return new FlatXmlDataSetBuilder().build(new StringReader(xml));
        } catch (DataSetException e) {
            throw new RuntimeException(e);
        }
    }
}
