package glg203.dao;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import org.h2.api.ErrorCode;

import glg203.model.Person;
import glg203.persistenceFramework.AbstractDAO;
import glg203.persistenceFramework.exception.CheckException;
import glg203.persistenceFramework.exception.DAOException;
import glg203.persistenceFramework.exception.DuplicateKeyException;
import glg203.persistenceFramework.exception.NoSuchElementException;
import glg203.persistenceFramework.exception.TechnicalSQLException;
import glg203.persistenceFramework.template.SQLSimpleHelper;

public class PersonDAO extends AbstractDAO<Person> {

    public PersonDAO(Connection connection) {
        super(connection);
    }

    /**
     * Saves p in the database.
     * <p>
     * p must not already exist here (with the same id).
     * <p>
     * post : p is saved in the database.
     * 
     * @param p
     * @throws DAOException if p is there
     */
    public void save(Person p) throws DuplicateKeyException, CheckException {
        String sql = "insert into Person values (?,?)";
        try {
            SQLSimpleHelper.update(getConnection(), sql, p.getId(), p.getName());
        } catch (SQLException e) {
            if (e.getErrorCode() == ErrorCode.DUPLICATE_KEY_1) {
                throw new DuplicateKeyException(e);
            } else if (e.getErrorCode() == ErrorCode.CHECK_CONSTRAINT_VIOLATED_1
                || e.getErrorCode() == ErrorCode.CHECK_CONSTRAINT_INVALID
                || e.getErrorCode() == ErrorCode.NULL_NOT_ALLOWED
            ) {
                throw new CheckException(e);
            } else {
                throw new TechnicalSQLException(e);
            }
        }
    }

    private Person extractPerson(ResultSet res) throws SQLException {
        Long id = res.getLong("id");
        String name = res.getString("name");
        return new Person(id, name);
    }

    public Person findById(Long id) {
        try {
            String sql = "select * from person where id = ?";
            List<Person> result = SQLSimpleHelper.query(getConnection(), sql, this::extractPerson, id);
            if (result.isEmpty())
                return null;
            else
                return result.get(0);
        } catch (SQLException e) {
            throw new TechnicalSQLException(e);
        }
    }

    public List<Person> findAll() {
        try {
            String sql = "select * from person";
            return SQLSimpleHelper.query(getConnection(), sql, this::extractPerson);
        } catch (SQLException e) {
            throw new TechnicalSQLException(e);
        }
    }

    public void remove(long id) throws NoSuchElementException {
        try {
            if (findById(id) == null) {
                throw new NoSuchElementException();
            }
            String sql = "delete from person where id = ?";
            SQLSimpleHelper.update(getConnection(), sql, id);
        } catch (SQLException e) {
            throw new TechnicalSQLException(e);
        }
    }

    public void update(Person p) throws NoSuchElementException, CheckException {
        try {
            if (findById(p.getId()) == null) {
                throw new NoSuchElementException();
            }
            String sql = "update person set name = ? where id = ?";
            SQLSimpleHelper.update(getConnection(), sql, p.getName(), p.getId());
        } catch (SQLException e) {
            if (e.getErrorCode() == ErrorCode.CHECK_CONSTRAINT_VIOLATED_1) {
                throw new CheckException(e);
            } else {
                throw new TechnicalSQLException(e);
            }
        }
    }

    @Override
    public void remove(Long id) throws NoSuchElementException {
        try {
            if (findById(id) == null)
                throw new NoSuchElementException();
            String sql = "delete from person where id = ?";
            SQLSimpleHelper.update(getConnection(), sql, id);
        } catch (SQLException e) {
            throw new TechnicalSQLException(e);
        }
    }
}
