package glg203.persistenceFramework.template;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Arrays;
import java.util.List;


/**
 * Un ensemble de méthodes pratiques pour exécuter des requêtes SQL.
 */
public class SQLSimpleHelper {
    
    /**
     * Helper class statique : pas d'instanciation.
     */
    private SQLSimpleHelper() {}

   
    /**
     * Exécute une requête de modification (insertion, update) dans la base.
     * 
     * <p>Exemple
     * <pre>
     * String sql = "insert into personne values (?,?)"
     * SQLSimpleHelper.update(connection, sql, 3, "Turing");
     * <pre>
     * exécute la requête :
     * <pre>
     * insert into personne values (3, 'Turing')
     * </pre>
     * 
     * 
     * @param sql  une requête de modification ; des '?' signalent des arguments.
     * @param args les valeurs des arguments.
     * @throws SQLException
     * @throws SQLTemplateException
     */
    public static void update(Connection connection, String sql, Object... args) throws SQLException {
        List<Object> argList = Arrays.asList(args);
        new SQLUpdateTemplate(connection, sql).execute(argList);

    }

    /**
     * Exécute une requête de type select et retourne le résultat.
     * <p> La fonction prend comme argument un {@link ResultSetExtractor} 
     * qui peut être implémenté par une <em>lambda expression</em> ; celui-ci devra
     * construire un objet du type voulu à partir du {@link ResultSet}.
     * <p>Exemple
     * <pre>
     *  String sql = "select * from person where id = ?";
     *  List<Person> result = SQLSimpleHelper.query(getConnection(), sql, 
     *                        rset -> extractPerson(rset), id);            
     * </pre>
     * 
     * La fonction <code>extractPerson</code> pourrait être quelque chose comme :
     * 
     * <pre>
     *  private Person extractPerson(ResultSet res) throws SQLException {
     *   Long id = res.getLong("id");
     *   String name = res.getString("name");
     *   return new Person(id, name);
     *  }
     * </pre>
     * 
     * Note à propos de l'interface : on ne peut pas utiliser ici les interfaces génériques de java 8 
     * (comme {@link java.util.function.Function}) parce que la méthode peut <em>a priori</em> lever une {@link SQLException}.
     * 
     * @param <T> Le type des objets à extraire
     * @param connection la connexion à la base
     * @param sql code sql de la requête
     * @param extractor la fonction d'extraction du résultat
     * @param args les arguments de la requête
     * @return
     * @throws SQLException
     * @throws SQLTemplateException
     */

    public static<T> List<T> query(Connection connection, String sql, 
        ResultSetExtractor<T> extractor, Object... args) throws SQLException {
        return new SQLQueryTemplate<>(connection, sql, extractor).execute(Arrays.asList(args));
    }

    /**
     * Exécute une requête qui doit retourner exactement un entier (ou null).
     * 
     * @param sql
     * @param args
     * @return un Integer ou null, selon que la requête a un retour ou non.
     * @throws SQLException
     * @throws SQLTemplateException si la requête retourne plus d'un résultat.
     */
    public static Integer sqlIntQuery(Connection connection, String sql, Object... args) throws SQLException {
        List<Integer> res = new SQLQueryTemplate<>(connection, sql, resultSet -> resultSet.getInt(1))
            .execute(Arrays.asList(args));
        if (res.isEmpty())
            return null;
        else if (res.size() == 1)
            return res.get(0);
        else
            throw new SQLException("la requête renvoie " + res.size() + " entiers " + res);
    }

    /**
     * Exécute une requête qui doit retourner exactement un long (ou null).
     * @param connection
     * @param sql
     * @param args
     * @return
     * @throws SQLException
     * @throws SQLTemplateException
     */
    public static Long sqlLongQuery(Connection connection, String sql, Object... args) throws SQLException {
        List<Long> res = new SQLQueryTemplate<Long>(connection, sql, resultSet -> resultSet.getLong(1))
            .execute(Arrays.asList(args));
        if (res.isEmpty())
            return null;
        else if (res.size() == 1)
            return res.get(0);
        else
            throw new SQLException("la requête renvoie " + res.size() + " entiers " + res);
    }
}