package glg203.persistenceFramework.template;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.List;

/**
 * Classe de base pour implémenter les Query et les Updates.
 * Utilise le pattern "template method".
 * 
 */
abstract class SQLStatementTemplate<T> {

    private final Connection connection;
    private final String sql;

    protected SQLStatementTemplate(Connection connection, final String sql) {
        this.connection = connection;        
        this.sql = sql;        
    }

    protected Connection getConnection() {
        return connection;
    }

    /**
     * Exécute la requête en appelant les méthodes abstraites à définir par le programmeur.
     * <p>Cette fonction est déclarée <code>final</code> parce qu'elle n'a plus à être redéfinie ;
     * elle décrit la suite des opérations à réaliser, en délégant le travail aux méthodes 
     * {@link #performStatement(Object, PreparedStatement)}, {@link #initResult()} et  {@link #performAtEnd(Connection, Object)} ;
     * 
     * <p> Seule {@link #performStatement(Object, PreparedStatement)} doit impérativement être redéfinie.
     * 
     */
    public final T execute(List<Object> argList) throws SQLException {
        try (PreparedStatement pst = getConnection().prepareStatement(sql)) {
            for (int i = 0; i < argList.size(); i++) {
                pst.setObject(i + 1, argList.get(i));
            }
            return performWithStatement(pst);
        }
    }

    /**
     * Exécute le preparedStatement pour la requête SQL passée en paramètre du
     * constructeur, initialisée avec un des t-uples d'arguments.
     * 
     * @param pst le preparedStatement à exécuter.
     * @return le résultat de la requête (Void/null si rien ne doit être retourné).
     * @throws SQLException
     */
    protected abstract T performWithStatement(PreparedStatement pst) throws SQLException;

}